package cn.shsmi.search;

import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;

import java.util.ArrayList;
import java.util.List;

import cn.shsmi.MapView;

/**
 * Created by George on 2017/3/1.
 */

public class MapSearch {

    /**
     * 地图视图
     */
    private MapView mapView;

    /**
     * 离线搜索数据库完整路径
     */
    private String sqlitePath;

    /**
     * 获取搜索实例
     * @param mapView 地图视图
     * @param sqlitePath 离线搜索数据库完整路径
     */
    public MapSearch(MapView mapView, String sqlitePath) {
        this.mapView = mapView;
        this.sqlitePath = sqlitePath;
        mapView.openGisDB(sqlitePath);
    }

    @Override
    protected void finalize() throws Throwable {
        super.finalize();
        mapView.closeGisDB();
    }

    /**
     * 进行地图搜索
     * @param searchStr 搜索字符串
     * @return json格式的字符串（详细属性信息）的数组
     */
    public List<String> search(String searchStr) {

        //创建返回值
        List<String> result = new ArrayList();

        //判断参数正确与否
        if (searchStr == null || searchStr.length() <= 0) {
            return result;
        }

        //打开数据库
        SQLiteDatabase database = SQLiteDatabase.openDatabase(sqlitePath, null, SQLiteDatabase.OPEN_READONLY);

        //表0
        Cursor cursor = rawQuery(database, "point_0", "id", searchStr);
        if (cursor != null) {
            while (cursor.moveToNext()) {
                String featureStr = findFeature(0, cursor.getString(0));
                if (featureStr != null && featureStr.length() >= 0) {
                    result.add(featureStr);
                }
            }
        }
        //表5
        cursor = rawQuery(database, "point_5", "OBJECTID", searchStr);
        if (cursor != null) {
            while (cursor.moveToNext()) {
                String featureStr = findFeature(5, cursor.getString(0));
                if (featureStr != null && featureStr.length() >= 0) {
                    result.add(featureStr);
                }
            }
        }
        //表12
        cursor = rawQuery(database, "point_12", "ANGEL", searchStr);
        if (cursor != null) {
            while (cursor.moveToNext()) {
                String featureStr = findFeature(12, cursor.getString(0));
                if (featureStr != null && featureStr.length() >= 0) {
                    result.add(featureStr);
                }
            }
        }
        //表13
        cursor = rawQuery(database, "point_13", "ANGEL", searchStr);
        if (cursor != null) {
            while (cursor.moveToNext()) {
                String featureStr = findFeature(13, cursor.getString(0));
                if (featureStr != null && featureStr.length() >= 0) {
                    result.add(featureStr);
                }
            }
        }
        //表14
        cursor = rawQuery(database, "point_14", "ANGEL", searchStr);
        if (cursor != null) {
            while (cursor.moveToNext()) {
                String featureStr = findFeature(14, cursor.getString(0));
                if (featureStr != null && featureStr.length() >= 0) {
                    result.add(featureStr);
                }
            }
        }
        //表16
        cursor = rawQuery(database, "point_16", "ANGEL", searchStr);
        if (cursor != null) {
            while (cursor.moveToNext()) {
                String featureStr = findFeature(16, cursor.getString(0));
                if (featureStr != null && featureStr.length() >= 0) {
                    result.add(featureStr);
                }
            }
        }
        //表17
        cursor = rawQuery(database, "point_17", "ANGEL", searchStr);
        if (cursor != null) {
            while (cursor.moveToNext()) {
                String featureStr = findFeature(17, cursor.getString(0));
                if (featureStr != null && featureStr.length() >= 0) {
                    result.add(featureStr);
                }
            }
        }
        //表18
        cursor = rawQuery(database, "point_18", "ANGEL", searchStr);
        if (cursor != null) {
            while (cursor.moveToNext()) {
                String featureStr = findFeature(18, cursor.getString(0));
                if (featureStr != null && featureStr.length() >= 0) {
                    result.add(featureStr);
                }
            }
        }
        //表20
        cursor = rawQuery(database, "point_20", "ANGEL", searchStr);
        if (cursor != null) {
            while (cursor.moveToNext()) {
                String featureStr = findFeature(20, cursor.getString(0));
                if (featureStr != null && featureStr.length() >= 0) {
                    result.add(featureStr);
                }
            }
        }
        //表21
        cursor = rawQuery(database, "point_21", "ANGEL", searchStr);
        if (cursor != null) {
            while (cursor.moveToNext()) {
                String featureStr = findFeature(21, cursor.getString(0));
                if (featureStr != null && featureStr.length() >= 0) {
                    result.add(featureStr);
                }
            }
        }

        //关闭数据库
        database.close();

        return result;
    }

    private String findFeature(int table, String id) {
        return mapView.getGisInfoFromDB(0, table, Integer.valueOf(id));
    }

    private Cursor rawQuery(SQLiteDatabase database, String table, String key, String keyValue) {
        //读取数据库
        String sql = "select " +
                key +
                " from " +
                table +
                " where name like '%" +
                keyValue +
                "%'";
        return database.rawQuery(sql, new String[]{});
    }

}
